--
-- MI3_Slash.lua
--
-- Handle all slash commands and the actions performed by slash commands.
-- All option dialog settings use slash commands for performing their
-- actions.
--
-- Note: version history now located in ReadMe.txt
--

local MI3_DeleteMode = ""

-- defaults for all MobInfo config options
local MI3_TTDefaults = {
	ShowHealth=1, ShowMana=0, ShowXp=1, ShowNo2lev=1, ShowKills=0, ShowLoots=1, ShowTotal=1, 
	ShowCoin=0, ShowIV=0, ShowEmpty=0, ShowLowHpAction=1, ShowCloth=1, ShowDamage=1,
	ShowDps=1, ShowLocation=1, ShowQuality=1, ShowResists=1, ShowImmuns=1, ShowItems=1,
	ShowClothSkin=1, ShowIGrey=0, ShowIWhite=1, ShowIGreen=1, ShowIBlue=1, ShowIPurple=1 } 


-----------------------------------------------------------------------------
-- MI3_SlashAction_Default()
--
-- Default tooltip content settings
--
function MI3_SlashAction_Default()
	local idx, def
	for idx,def in pairs(MI3_TTDefaults) do
		MobInfoConfig[idx] = def
	end
end -- MI3_SlashAction_Default


-----------------------------------------------------------------------------
-- MI3_SlashAction_AllOn()
--
-- Show all tooltip content
--
function MI3_SlashAction_AllOn()
	local idx,def
	for idx,def in pairs(MI3_TTDefaults) do
		MobInfoConfig[idx] = 1
	end
end -- MI3_SlashAction_AllOn


-----------------------------------------------------------------------------
-- MI3_SlashAction_AllOff()
--
-- Show no extra info in tooltip
--
function MI3_SlashAction_AllOff()
	local idx,def
	for idx,def in pairs(MI3_TTDefaults) do
		MobInfoConfig[idx] = 0
	end
end -- MI3_SlashAction_AllOff


-----------------------------------------------------------------------------
-- MI3_RegisterWithAddonManagers()
--
-- Register MobInfo with the KHAOS AddOn manager. This is a very simple
-- registration that merely creates a button to open the MobInfo options
-- dialog.
--
-- Register MobInfo with the myAddons manager.
--
-- Register with the EARTH AddOn manager.
--
function MI3_RegisterWithAddonManagers()

	-- register with myAddons manager
	if ( myAddOnsFrame_Register ) then
		local mobInfo2Details = {
		name = "MobInfo",
		version = miVersionNo,
		author = "Skeeve & Dizzarian (Перевод: StingerSoft)",
		website = "http://www.dizzarian.com/forums/viewforum.php?f=16",
		category = MYADDONS_CATEGORY_OTHERS,
		optionsframe = "MI3_OptionsFrame"
		}
		myAddOnsFrame_Register( mobInfo2Details )
	end

	-- register with EARTH manager (mainly for Cosmos support)
	if EarthFeature_AddButton then
		EarthFeature_AddButton(
			{
				id = "MobInfo",
				name = "MobInfo",
				subtext = "v"..miVersionNo,
				tooltip = MI_DESCRIPTION,
				icon = "Interface\\CharacterFrame\\TemporaryPortrait-Monster",
				callback = function(state) MI3_SlashParse("") end
			}
		)
		if not MobInfoConfig.ShowMMButton then MobInfoConfig.ShowMMButton = 0 end

	-- register with KHAOS (only if EARTH not found)
	elseif Khaos then
		Khaos.registerOptionSet(
			"tooltip",
			{
				id = "MobInfoOptionSet",
				text = "MobInfo",
				helptext = MI_DESCRIPTION,
				difficulty = 1,
				callback = function(state) end,
				default = true,
				options = {
					{
						id = "MobInfoOptionsHeader",
						type = K_HEADER,
						difficulty = 1,
						text = MI_TXT_WELCOME,
						helptext = MI_DESCRIPTION
					},
					{
						id = "MobInfoOptionsButton",
						type = K_BUTTON,
						difficulty = 1,
						text = "MobInfo",
						helptext = "",
						callback = function(state) MI3_SlashParse("") end,
						feedback = function(state) end,
						setup = { buttonText = MI_TXT_OPEN }
					}
				}
			}
		)
		if not MobInfoConfig.ShowMMButton then MobInfoConfig.ShowMMButton = 0 end
	end
end  -- MI3_RegisterWithAddonManagers()


-----------------------------------------------------------------------------
-- MI3_SlashAction_ClearTarget()
--
-- Clear MobInfo and MobHealth data for current target.
--
function MI3_SlashAction_ClearTarget()
	local index = MI3_Target.mobIndex
	if index then
		MI3_MobHealth_ClearTargetData()
		MI3_DeleteMobData( index )
		MI3_Target = {}
		MI3_OnTargetChanged()
		MI3_DbOptionsFrameOnShow()
		chattext( "данные по цели "..MI_Green..index..MI_White.." удалены" )
	end
end  -- MI3_SlashAction_ClearTarget()


-----------------------------------------------------------------------------
-- MI3_Slash_ClearDbConfirmed()
--
-- Clear database handler : clear specific database if reqzested and 
-- confirmed by user.
--
local function MI3_Slash_ClearDbConfirmed()
	if MI3_DeleteMode == "MobDb" then
		local curZoneName = MI3_ZoneTable[MI3_CurZone]
		MI3_DeleteAllMobData()
		MI3_ZoneTable[MI3_CurZone] = curZoneName
		MobInfoConfig.ImportSignature = ""
	elseif MI3_DeleteMode == "HealthDb" then
		MI3_MobHealth_Reset()
	elseif MI3_DeleteMode == "PlayerDb" then
		MobHealthPlayerDB = {}
	end
	chattext( "база данных удалена: "..MI3_DeleteMode )
	MI3_Target = {}
	MI3_OnTargetChanged()
	MI3_DbOptionsFrameOnShow()
	collectgarbage( "collect" )
end  -- MI3_Slash_ClearDbConfirmed()


-----------------------------------------------------------------------------
-- MI3_SlashAction_ClearHealthDb()
--
-- Clear entire contents of MobInfo and MobHealth databases.
-- Ask for confirmation before performing the clear operation.
--
function MI3_SlashAction_ClearHealthDb()
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].text = MI_TXT_CLR_ALL_CONFIRM.."'"..MI3_OPTIONS[this:GetName()].help.."' ?"
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].OnAccept = MI3_Slash_ClearDbConfirmed
	MI3_DeleteMode = "HealthDb"
	local dialog = StaticPopup_Show( "MOBINFO_CONFIRMATION", "")
end  -- MI3_SlashAction_ClearHealthDb()


-----------------------------------------------------------------------------
-- MI3_SlashAction_ClearPlayerDb()
--
-- Clear entire contents of MobInfo and MobHealth databases.
-- Ask for confirmation before performing the clear operation.
--
function MI3_SlashAction_ClearPlayerDb()
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].text = MI_TXT_CLR_ALL_CONFIRM.."'"..MI3_OPTIONS[this:GetName()].help.."' ?"
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].OnAccept = MI3_Slash_ClearDbConfirmed
	MI3_DeleteMode = "PlayerDb"
	local dialog = StaticPopup_Show( "MOBINFO_CONFIRMATION", "")
end  -- MI3_SlashAction_ClearPlayerDb()


-----------------------------------------------------------------------------
-- MI3_SlashAction_ClearMobDb()
--
-- Clear entire contents of MobInfo and MobHealth databases.
-- Ask for confirmation before performing the clear operation.
--
function MI3_SlashAction_ClearMobDb()
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].text = MI_TXT_CLR_ALL_CONFIRM.."'"..MI3_OPTIONS[this:GetName()].help.."' ?"
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].OnAccept = MI3_Slash_ClearDbConfirmed
	MI3_DeleteMode = "MobDb"
	local dialog = StaticPopup_Show( "MOBINFO_CONFIRMATION", "")
end  -- MI3_SlashAction_ClearMobDb()


-----------------------------------------------------------------------------
-- MI3_Slash_TrimDownConfirmed()
--
-- Trim down the contents of the mob info database by removing all data
-- that is not set as being recorded. This function is called when the
-- user confirms the delete confirmation.
--
local function MI3_Slash_TrimDownConfirmed()
	-- loop through database and check each record
	-- remove all fields within the record where recording of the field is disabled
	for idx, mobInfo in pairs(MobInfoDB) do
		if  MobInfoConfig.SaveBasicInfo == 0 then
			mobInfo.bi = nil
			mobInfo.ml = nil
		end
		if  MobInfoConfig.SaveQualityData == 0 then
			mobInfo.qi = nil
		end
		if  MobInfoConfig.SaveItems == 0 then
			mobInfo.il = nil
		end
		if  MobInfoConfig.SaveResist == 0 then
			mobInfo.re = nil
		end
		if  MobInfoConfig.SaveCharData == 0 then
			MI3_RemoveCharData( mobInfo )
		end
	end

	if  MobInfoConfig.SaveItems == 0 then
		MI3_ItemNameTable = {}
	end

	-- char table can be deleted when not saving char specific data
	if  MobInfoConfig.SaveCharData == 0 then
		MI3_CharTable = { charCount = 0 }
	end

	-- force a cleanup after trimming down
	MI3_ClearMobCache()
	MI3_CheckAndCleanDatabases()
	collectgarbage( "collect" )

	MI3_DbOptionsFrameOnShow()
end -- MI3_Slash_TrimDownConfirmed()


-----------------------------------------------------------------------------
-- MI3_SlashAction_TrimDownMobData()
--
-- Trim down the contents of the mob info database by removing all data
-- that is not set as being recorded. Ask for a confirmation before
-- actually deleting anything.
--
function MI3_SlashAction_TrimDownMobData()
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].text = MI_TXT_TRIM_DOWN_CONFIRM
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].OnAccept = MI3_Slash_TrimDownConfirmed
	local dialog = StaticPopup_Show( "MOBINFO_CONFIRMATION", "")
end  -- MI3_SlashAction_TrimDownMobData()


-----------------------------------------------------------------------------
-- MI3_UpdateMob()
--
-- Update a specific existing mob by adding to it the given new Mob data.
--
local function MI3_UpdateMob( mobIndex, newMobInfo )
	local existingMobInfo = MobInfoDB[mobIndex]
	local existingMobData, newMobData = {}, {}
	MI3_GetMobDataFromMobInfo( existingMobInfo, existingMobData )
	MI3_GetMobDataFromMobInfo( newMobInfo, newMobData )
	MI3_AddTwoMobs( existingMobData, newMobData )
	MI3_StoreAllMobData( existingMobData, nil, nil, MI3_PlayerName, mobIndex )
end -- MI3_UpdateMob()

-----------------------------------------------------------------------------
-- MI3_AdaptImportLocation()
--
-- Adapt the location info of an imported Mob. This is only necessary for
-- Mobs in instances, because instances are not available in the WoW
-- zone tables.
--
local function MI3_AdaptImportLocation( mobInfo, importZoneTable )
	-- decode mob location information
	local mobData = {}
	MI3_DecodeMobLocation( mobInfo, mobData )
	local loc = mobData.location
	if not loc or not loc.z then return end

	-- find correct zone name
	local zone = mobData.location.z
	local zoneName = importZoneTable[zone]

	-- update the mobs location info
	if zoneName and zoneName ~= "" then
		MI3_SetNewZone( zoneName )
		importZoneTable[MI3_CurZone] = zoneName
		mobInfo.ml = (loc.x1 or "").."/"..(loc.y1 or "").."/"..(loc.x2 or "").."/"..(loc.y2 or "").."//"..MI3_CurZone
	else
		mobInfo.ml = nil
	end
end -- MI3_AdaptImportLocation()


-----------------------------------------------------------------------------
-- MI3_SlashAction_ImportMobData()
--
-- Import externally supplied MobInfo database into own database.
--
function MI3_SlashAction_ImportMobData()
	local newMobs, updatedMobs, newHealth, newItems = 0, 0, 0, 0
	local mobIndex, mobInfo, healthInfo
	local oldCurZone = MI3_CurZone

	chattext( " начало импорта внешней базы данных ...." )

	-- import loot items into main loot item database
	for itemId, itemInfo in pairs(MI3_ItemNameTable_Import) do
		if not MI3_ItemNameTable[itemId] then
			MI3_ItemNameTable[itemId] = itemInfo
			newItems = newItems + 1
		end
	end

	-- import health data into main Mob health database
	for mobIndex, newHealthData in pairs(MobHealthDB_Import) do
	    local oldHealthData = MobHealthDB[mobIndex]
		if not oldHealthData then
			MobHealthDB[mobIndex] = newHealthData
			newHealth = newHealth + 1
		else
		    local _, oldPct = MI3_HpDecode( oldHealthData )
		    local _, newPct = MI3_HpDecode( newHealthData )
		    if newPct and oldPct and newPct > 90 and newPct > oldPct then
				MobHealthDB[mobIndex] = newHealthData
				newHealth = newHealth + 1
		    end
		end
	end

	-- swap name/id in zone name table
	if MI3_ZoneTable_Import.cnt then
		local zoneName, zoneId
		local newTable = {}
		MI3_ZoneTable_Import.cnt = nil
		for zoneName, zoneId in pairs(MI3_ZoneTable_Import) do
			newTable[zoneId] = zoneName
		end
		MI3_ZoneTable_Import = newTable
	end

	-- import Mobs into main Mob database
	for mobIndex, mobInfo in pairs(MobInfoDB_Import) do
		MI3_RemoveCharData( mobInfo )
		MI3_AdaptImportLocation( mobInfo, MI3_ZoneTable_Import )
		if MobInfoDB[mobIndex] then
			updatedMobs = updatedMobs + 1
			if MobInfoConfig.ImportOnlyNew == 0 then
				-- import Mob that already exists
				MI3_UpdateMob( mobIndex, mobInfo )
			end
		else
			-- import unknown Mob
			MobInfoDB[mobIndex] = mobInfo
			newMobs = newMobs + 1
		end
	end

	-- update item cross reference table after import
	if MobInfoConfig.ImportOnlyNew == 0 then
		MI3_BuildXRefItemTable()
	end

	-- restore current zone ID after import
	MI3_CurZone = oldCurZone

	chattext( " импортированно "..newMobs.." новых существ" )
	chattext( " импортированно "..newHealth.." новых значений здоровья" )
	chattext( " импортированно "..newItems.." повых добытых предметов" )
	if MobInfoConfig.ImportOnlyNew == 0 then
		chattext( " обновлены данные "..updatedMobs.." существующих существ" )
	else
		chattext( " НЕ обновлены данные: "..updatedMobs.." существующих существ" )
	end

	-- update database options frame
	MobInfoConfig.ImportSignature = MI3_Import_Signature
	MI3_DbOptionsFrameOnShow()
end  -- MI3_SlashAction_ImportMobData()


-----------------------------------------------------------------------------
-- MI3_SlashAction_DeleteSearch()
--
-- Delete all Mobs in the search result list from the MobInfo database.
-- This function will ask for confirmation before deleting.
--
function MI3_SlashAction_DeleteSearch()
	local confirmationText = string.format( MI_TXT_DEL_SEARCH_CONFIRM, MI3_NumMobsFound )
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].text = confirmationText
	StaticPopupDialogs["MOBINFO_CONFIRMATION"].OnAccept = MI3_DeleteSearchResultMobs
	local dialog = StaticPopup_Show( "MOBINFO_CONFIRMATION", "")
end -- MI3_SlashAction_DeleteSearch()


-----------------------------------------------------------------------------
-- MI3_SlashInit()
--
-- Add all Slash Commands
--
function MI3_SlashInit()
	SlashCmdList["MOBINFO"] = MI3_SlashParse
	SLASH_MOBINFO1 = "/mobinfo" 
	SLASH_MOBINFO2 = "/mobinfo3" 
	SLASH_MOBINFO3 = "/mi3" 
end  -- MI3_SlashInit()


-----------------------------------------------------------------------------
-- MI3_SlashParse()
--
-- Parses the msg entered as a slash command. This function is also used
-- for the internal purpose of setting all options in the options dialog.
-- When used by the options dialog there is no need to actually update the
-- dialog, which is indicated by the "updateOptions" parameter.
--
function MI3_SlashParse( msg )
	-- extract option name and option argument from message string
	local _, _, cmd, param = string.find( string.lower(msg), "([%w_]*)[ ]*([-%w]*)") 
	
	-- handle all simple commands that dont require parsing right here
	-- handle show/hide of options dialog first of all
	if  not cmd  or  cmd == ""  or  cmd == "config"  then
		if MI3_OptionsFrame:IsVisible() then
			MI3_OptionsFrame:Hide()
		else
			MI3_OptionsFrame:Show()
		end
		return
	elseif cmd == 'version' then
		chattext( ' MobInfo-2 Версия '..miVersionNo )
		return
	elseif cmd == 'update' and MI3_UpdatePrices then
		MI3_UpdatePrices()
		return
	elseif cmd == 'help' then
		chattext( ' Использование: введите /mobinfo3 или /mi3 для открытия окна настроек' )
		chattext( ' Все вопросы '..MI_LightGreen..'http://www.dizzarian.com/forums/viewforum.php?f=16' )
		return
	end

	-- search for the option data structure matching the command
	local optionName, optionData
	for idx, val in pairs(MI3_OPTIONS) do
		local lower_opt = string.lower( idx )
		local optionCommand = string.sub(lower_opt, 8)
		if cmd == lower_opt or cmd == optionCommand then
			optionName = string.sub(idx, 8)
			optionData = val
			break
		end
	end

	-- now call the option handler for the more complex commands
	if  optionData  then
		MI3_OptionParse( optionName, optionData, param )
	end
end -- of MI3_SlashParse()


-----------------------------------------------------------------------------
-- MI3_OptionParse()
--
-- Parses the more complex option toggle/set commands. There are 4
-- categories of options:
--   * options that can toggle between an on and off state
--   * options that represent a numeric value
--   * options that represent a text
--   * options that activate a special functionality represented by a
--     handler function that must correspond to a specific naming convention
--
function MI3_OptionParse( optionName, optionData, param )
	-- handle the option according to its option type: its either a
	-- switch being toggleg, a value being set, or a special action
	if optionData.val then
		-- it is a slider setting a value
		-- get new option value from parameter and set it
		local optValue = tonumber( param ) or 0
		MobInfoConfig[optionName] = optValue
		if  updateOptions  then
			chattext( optionData.text.." : "..MI_Green..optValue )
		end

	elseif optionData.txt then
		-- it is a text based option
		MobInfoConfig[optionName] = param
		if  updateOptions  then
			chattext( optionData.text.." : "..MI_Green..param )
		end

	elseif  MobInfoConfig[optionName]  then
		-- it is a switch toggle option:
		-- get current option value and toggle it to the opposite state (On<->Off)
		local valTxt = { val0 = "-OFF-",  val1 = "-ON-" }
		local optValue = MobInfoConfig[ optionName ]
		optValue = 1 - optValue  -- toggle option
		MobInfoConfig[optionName] = optValue
		if optionData.text and optionData.text ~= "" then
			chattext( optionData.text.." : "..MI_Green..valTxt["val"..optValue] )
		end

		-- some toggle switches control recording options which in turn controls events
		MI3_InitializeEventTable()
	else
		-- special action commands have a corresponding handler function
		local actionHandlerName = "MI3_SlashAction_"..optionName
		local actionHandler = getglobal( actionHandlerName )
		if  actionHandler  then
			actionHandler()
			updateOptions = true -- for AllOn, AllOff, etc.
		end
	end

	-- update position and visibility of minimap button
	-- (80/80 is circle radius, 50/52 is circle center)
	if optionName == "MMButtonPos" then
		local pos = MobInfoConfig.MMButtonPos
		MI3_MinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT",50 - (80 * cos(pos)), (80 * sin(pos)) - 52 )
	end
	if MobInfoConfig.ShowMMButton == 1 then
		MI3_MinimapButton:Show()
	else
		MI3_MinimapButton:Hide()
	end

	-- update everything that might depend on config actions
	MI3_MobHealth_SetPos()
	MI3_UpdateOptions()
	MI3_SetupTooltip()
end  -- MI3_OptionParse()
